import {
  Modal, ModalService, Select, GridEditable, Iterable, Button,
  AlertService, DateRange, Grid, Text, Textarea,
} from '@zeedhi/common';
import {
  Http, IDictionary, IEventParam, Metadata,
} from '@zeedhi/core';

import { IterableComponentRender } from '@zeedhi/teknisa-components-common';

import { getRouteParams } from '../router';

export class ApontamentoController {
  public dataSelects: IDictionary = {
    selectUsuario: [],
    selectSetor: [],
  };

  public parametroBusca: IDictionary = [];

  public apontamentos: IDictionary = [];

  public valorHora: String = '';

  public dataReferencia: String = '';

  public textoModal: String = '';

  public nrfuncionario: Number = 0;

  public nrapontamento: Number = 0;

  public ausencia: string = '';

  public valueIdAlmoco: Number = 0;

  public qtdRegistroAlmoco: number = 0;

  public horasTotais: string = '';

  public dataFilter: IDictionary = {
    NRFUNCIONARIO: '',
    NRSETOR: '',
    DTULTATU: [],
  };

  private modalApontamento!: Modal;

  private modalDetalhes!: Modal;

  public async criarModalApontamento() {
    this.modalApontamento = await ModalService.createFromJson('./modal/modalNovoApontamento', true);
  }

  public async criarModalDetalhes() {
    this.modalDetalhes = await ModalService.createFromJson('./modal/modalDetalhesApontamento', true);
  }

  public destruirModalApontamento() {
    this.modalApontamento?.destroy();
  }

  public destroyModalDetalhes() {
    this.modalDetalhes?.destroy();
  }

  public abrirModalApontamento() {
    this.modalApontamento.show();
  }

  public abrirModalDetalhes({ row }: IEventParam<Button>) {
    this.nrapontamento = row.id.split('-');
    this.modalApontamento.destroy();
    this.modalDetalhes.show();
    this.criarModalApontamento();
  }

  public fecharModalApontamento() {
    this.modalApontamento.hide();
    this.modalApontamento.destroy();
    this.criarModalApontamento();
  }

  public fecharModalDetalhes() {
    this.modalDetalhes.hide();
    this.modalDetalhes.destroy();
    this.criarModalDetalhes();
  }

  public carregaDadosFiltros() {
    this.getDadosUsuario();
    this.getDadosSetor();
  }

  public setDataSelects({ component }: IEventParam<Select>) {
    component.datasource.data = this.dataSelects[component.name];
  }

  public setData(nm: string, id: number) {
    return {
      name: nm,
      value: id,
    };
  }

  public getDadosUsuario() {
    Http.get('/Employee').then((response: IDictionary) => {
      response.data.dataset.employee.forEach((dados: IDictionary) => {
        this.dataSelects.selectUsuario.push(this.setData(dados.NMFUNCIONARIO, dados.NRFUNCIONARIO));
      });
    });
  }

  public getDadosSetor() {
    Http.get('/getSector').then((response: IDictionary) => {
      response.data.dataset.options.forEach((dados: IDictionary) => {
        this.dataSelects.selectSetor.push(this.setData(dados.NMSETOR, dados.NRSETOR));
      });
    });
  }

  public getDadosCardApontamento() {
    let dataAtualizacao = '';
    const selectObjectItareble = Metadata.getInstance<IterableComponentRender>('iterable');
    Http.post('/getDadosApontamentos', this.dataFilter).then((response: IDictionary) => {
      response.data.dataset.dados.forEach((dados: IDictionary) => {
        if (dados.DATA === null) {
          dataAtualizacao = 'Não há apontamentos';
        } else {
          dataAtualizacao = `Último Apontamento: ${dados.DATA}`;
        }
        selectObjectItareble.datasource.data.push({
          id: dados.NRFUNCIONARIO,
          name: dados.NMFUNCIONARIO,
          data: dataAtualizacao,
        });
      });
    });
  }

  public getParamURL() {
    const routeParams = getRouteParams();
    const { nr } = routeParams;
    const nrfuncionario = parseInt(nr, 10);
    this.nrfuncionario = nrfuncionario;
  }

  public getDadosFiltros({ component }: IEventParam<any>) {
    let valorComponente = component.value;
    if (component.value === null) {
      valorComponente = '';
    }
    if (component.name === 'selectUsuario') {
      this.dataFilter.NRFUNCIONARIO = valorComponente;
      this.reloadItarable();
      this.getDadosCardApontamento();
    } else if (component.name === 'selectSetor') {
      this.dataFilter.NRSETOR = valorComponente;
      this.reloadItarable();
      this.getDadosCardApontamento();
    } else {
      this.dataFilter.DTULTATU = valorComponente;
      this.reloadItarable();
      this.getDadosCardApontamento();
    }
  }

  public reloadItarable() {
    const selectObjectItareble = Metadata.getInstance<IterableComponentRender>('iterable');
    selectObjectItareble.reload();
  }

  public recarregarGridModalNovoApontamento({ component }: IEventParam<Button>) {
    (component.parent as Iterable).reload();
  }

  public geraConteudoGrid() {
    const data = Metadata.getInstance<DateRange>('periodo').value;
    const selectObjectGrid = Metadata.getInstance<Grid>('grid');
    this.apontamentos = [];
    selectObjectGrid.reload();
    if (data.length > 1) {
      const dtInicio = this.criarNovoObjetoData(data[0]);
      const dtFim = this.criarNovoObjetoData(data[1]);
      let novaData = dtInicio;
      const NRFUNCIONARIO = this.nrfuncionario;
      this.formataData(novaData);
      let contador = 1;
      while (novaData < dtFim) {
        novaData = new Date(novaData.getTime() + (24 * 60 * 60 * 1000 - 1));
        let finalDeSemana = 0;
        const diaSemana = novaData.getDay();
        if (diaSemana === 0 || diaSemana === 6) {
          finalDeSemana = 1;
        } else {
          finalDeSemana = 0;
        }
        this.apontamentos.push({
          id: contador,
          data: this.formataData(novaData),
          funcionario: this.nrfuncionario,
          entrada: '',
          intervalo: '',
          saida: '',
          hrs_trabalhada: '00:00',
          ausencia: '00:00',
          horas_totais: '00:00',
          falta: '-',
          obs: false,
          finalSemana: finalDeSemana,
        });
        contador += 1;
      }
      Http.post('/buscaRegistrosHora', { NRFUNCIONARIO }).then((dados: IDictionary) => {
        const registros = dados.data.dataset.dados;
        registros.forEach((registrosApontamentos: IDictionary) => {
          Object.keys(this.apontamentos).forEach((t: any) => {
            if (this.apontamentos[t].data === registrosApontamentos.DATA) {
              if (registrosApontamentos.DSOBSREGISTRO !== null) {
                this.apontamentos.splice(this.apontamentos[t], this.apontamentos[t] = {
                  id: `${registrosApontamentos.NRAPONTAMENTO}-${registrosApontamentos.DATA}`,
                  data: registrosApontamentos.DATA,
                  funcionario: this.nrfuncionario,
                  entrada: registrosApontamentos.HRENTRADA,
                  intervalo: registrosApontamentos.HRINTERVALO,
                  saida: registrosApontamentos.HRSAIDA,
                  hrs_trabalhada: registrosApontamentos.HRTRABALHADAS,
                  ausencia: registrosApontamentos.HRAUSENCIA,
                  horas_totais: registrosApontamentos.HRTOTAIS,
                  falta: registrosApontamentos.IDFALTA,
                  obs: true,
                });
              } else {
                this.apontamentos.splice(this.apontamentos[t], this.apontamentos[t] = {
                  id: `${registrosApontamentos.NRAPONTAMENTO}-${registrosApontamentos.DATA}`,
                  data: registrosApontamentos.DATA,
                  funcionario: this.nrfuncionario,
                  entrada: registrosApontamentos.HRENTRADA,
                  intervalo: registrosApontamentos.HRINTERVALO,
                  saida: registrosApontamentos.HRSAIDA,
                  hrs_trabalhada: registrosApontamentos.HRTRABALHADAS,
                  ausencia: registrosApontamentos.HRAUSENCIA,
                  horas_totais: registrosApontamentos.HRTOTAIS,
                  falta: registrosApontamentos.IDFALTA,
                  obs: false,
                });
              }
            }
          });
        });
        selectObjectGrid.datasource.data = this.apontamentos as any;
      });
    }
  }

  public criarNovoObjetoData(data: string) {
    const arr = data.split('/');
    const invertePosicao = arr.reverse();
    const formata = invertePosicao.join('/');
    const dataForm = new Date(formata);
    return dataForm;
  }

  public formataData(novaData: Date) {
    let dataFormatada = '';
    if (novaData.getDate() < 10) {
      const diaFormatado = `0${novaData.getDate()}`;
      if ((novaData.getMonth() + 1) < 10) {
        const mesFormatado = `0${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      } else {
        const mesFormatado = `${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      }
    } else {
      const diaFormatado = novaData.getDate();
      if ((novaData.getMonth() + 1) < 10) {
        const mesFormatado = `0${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      } else {
        const mesFormatado = `${novaData.getMonth() + 1}`;
        dataFormatada = `${diaFormatado}/${mesFormatado}/${novaData.getFullYear()}`;
      }
    }
    return dataFormatada;
  }

  public linhaClicada({ component }: any) {
    this.dataReferencia = component.datasource.currentRow.data;
    if (component.datasource.currentRow.entrada === '') {
      this.parametroBusca = [];
      this.parametroBusca.push({
        NRAPONTAMENTO: -1,
      });
      this.abrirModalApontamento();
    } else {
      const nr = component.datasource.currentRow.id.split('-');
      this.parametroBusca = [];
      this.parametroBusca.push({
        NRAPONTAMENTO: nr[0],
      });
      this.abrirModalApontamento();
    }
  }

  public async getDadosApontamentoInserido() {
    this.qtdRegistroAlmoco = 0;
    this.valueIdAlmoco = 0;
    const arrayAuxiliar: IDictionary = [];
    const selectObjectGrid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    selectObjectGrid.datasource.data = [];
    await Http.post('/getDadosApontamentoInserido', this.parametroBusca).then((response: IDictionary) => {
      let valor = '';
      let tipo = '';
      let dt = '';
      let almoco = false;
      let valueAlmoco = 0;
      let objDate = '';
      const { dados } = response.data.dataset;
      dados.forEach((t: any) => {
        if (t.DTHRSAIDA === null && t.IDALMOCO === 'N') {
          if (t.DTHRENTRADA !== null && t.IDALMOCO === 'N') {
            valor = t.DTHRENTRADA.slice(11, 17);
            dt = t.DTHRENTRADA.slice(0, 10);
            tipo = '1';
            almoco = false;
            valueAlmoco = 0;
            objDate = this.criarObjetoDate(t.DTHRENTRADA);
          }
        } else if (t.DTHRSAIDA === null && t.IDALMOCO === 'S') {
          if (t.DTHRENTRADA !== null && t.IDALMOCO === 'S') {
            valor = t.DTHRENTRADA.slice(11, 17);
            dt = t.DTHRENTRADA.slice(0, 10);
            tipo = '1';
            almoco = false;
            valueAlmoco = 1;
            objDate = this.criarObjetoDate(t.DTHRENTRADA);
          }
        } else if (t.DTHRSAIDA !== null && t.IDALMOCO === 'S') {
          if (t.DTHRENTRADA === null && t.IDALMOCO === 'S') {
            valor = t.DTHRSAIDA.slice(11, 17);
            dt = t.DTHRSAIDA.slice(0, 10);
            tipo = '2';
            almoco = true;
            valueAlmoco = 1;
            this.qtdRegistroAlmoco += 1;
            objDate = this.criarObjetoDate(t.DTHRSAIDA);
          }
        } else if (t.DTHRSAIDA !== null && t.IDALMOCO === 'N') {
          if (t.DTHRENTRADA === null && t.IDALMOCO === 'N') {
            valor = t.DTHRSAIDA.slice(11, 17);
            dt = t.DTHRSAIDA.slice(0, 10);
            tipo = '2';
            almoco = false;
            valueAlmoco = 0;
            objDate = this.criarObjetoDate(t.DTHRSAIDA);
          }
        }
        const newRow = {
          id: t.NRREGISTROHORAS,
          enable_edit: false,
          enable_Save: false,
          enable_Check: almoco,
          name: t.NRREGISTROHORAS,
          tipoEntrada: tipo,
          idAlmoco: t.IDALMOCO,
          data: dt,
          funcionario: this.nrfuncionario,
          hora: valor,
          buttonSave: t.NRREGISTROHORAS,
          btnDelete: t.NRREGISTROHORAS,
          check: t.NRREGISTROHORAS,
          intervalo: valueAlmoco,
          habDesabCheck: almoco,
          paramOrdenarArray: objDate,
        };
        arrayAuxiliar.push(newRow);
      });
      const retornoOrdenado = arrayAuxiliar.sort((obj1: any, obj2: any) => {
        if (obj1.paramOrdenarArray > obj2.paramOrdenarArray) {
          return 1;
        }
        if (obj1.paramOrdenarArray < obj2.paramOrdenarArray) {
          return -1;
        }
        return 0;
      });
      selectObjectGrid.datasource.data = retornoOrdenado;
    });
    this.mostraTotalHorasModal();
    this.controlaVisibilidadeApurar();
  }

  public async salvarApontamento({ row }: IEventParam<Button>) {
    let arrayAuxiliar: IDictionary = [];
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    const linhaEditando = grid.getEditedRows();
    const conditionsRow = linhaEditando.find((linhaEditada) => linhaEditada.id === row.id) || row;
    if (conditionsRow.hora === '' && conditionsRow.tipoEntrada === '') {
      const tipo = 'warning';
      const texto = 'Favor preencher todos os campos!';
      this.mensagemAlert(texto, tipo);
    } else {
      conditionsRow.enable_Save = true;
      grid.reapplyConditions(conditionsRow);
      await grid.saveEditedRows();
      grid.datasource.data.forEach((obj: any) => {
        if (obj.name === row.name) {
          arrayAuxiliar = {
            DATA: obj.data,
            HORA: obj.hora,
            IDALMOCO: this.valueIdAlmoco,
            NRFUNCIONARIO: obj.funcionario,
            IDTIPORESGISTRO: obj.tipoEntrada,
          };
          Http.post('/insertNovoApontamento', arrayAuxiliar).then((response: any) => {
            const { retorno } = response.data.dataset.dados;
            const resposta = retorno.split('-');
            if (resposta[0] === 'OK') {
              const texto = 'Apontamento inserido com sucesso!';
              const sucesso = 'success';
              this.mensagemAlert(texto, sucesso);
              this.parametroBusca = [];
              this.parametroBusca.push({
                NRAPONTAMENTO: resposta[1],
              });
              this.getDadosApontamentoInserido();
            } else {
              const texto = 'Ocorreu um erro ao inserir o apontamento';
              const sucesso = 'error';
              this.mensagemAlert(texto, sucesso);
            }
            this.geraPrimeiroUltimoDiaMes();
            this.geraConteudoGrid();
          });
          this.mostraTotalHorasModal();
        }
      });
    }
  }

  public VisibilidadeSave({ row }: any) {
    return row.enable_Save;
  }

  public controlaVisibilidadeCheckAlmoco({ row }: any) {
    return row.enable_Check;
  }

  public habiDesabilitaCheckAlmoco({ row }: any) {
    return row.habDesabCheck;
  }

  public insereValorCheckBox({ row }: any) {
    return row.intervalo;
  }

  public buttonVisible = true;

  public testRowIsEditable({ row }: { row: IDictionary<any> }) {
    return row.enable_edit;
  }

  public addNovaBatida({ component }: any) {
    const grid: GridEditable = component.parent;
    const newId = grid.datasource.data.length;
    if (grid.datasource.data.length === 0) {
      const newRow = {
        id: `${newId}`,
        enable_edit: true,
        enable_Save: true,
        enable_Check: false,
        name: `${newId}`,
        tipoEntrada: '1',
        data: this.dataReferencia,
        funcionario: this.nrfuncionario,
        hora: '',
        intervalo: 0,
      };
      grid.addNewRow(newRow);
    } else if (grid.datasource.data.length > 0) {
      if (this.qtdRegistroAlmoco === 1) {
        const newRow = {
          id: `${newId}`,
          enable_edit: true,
          enable_Save: true,
          enable_Check: false,
          name: `${newId}`,
          tipoEntrada: '',
          data: this.dataReferencia,
          funcionario: this.nrfuncionario,
          hora: '',
          intervalo: 0,
        };
        grid.addNewRow(newRow);
      } else {
        const newRow = {
          id: `${newId}`,
          enable_edit: true,
          enable_Save: true,
          enable_Check: true,
          name: `${newId}`,
          tipoEntrada: '',
          data: this.dataReferencia,
          funcionario: this.nrfuncionario,
          hora: '',
          intervalo: 0,
        };
        grid.addNewRow(newRow);
      }
    }
  }

  public mensagemAlert(text: string, tipo: string) {
    AlertService.show({
      name: 'basic-alert',
      text: `${text}`,
      top: true,
      left: false,
      timeout: 3000,
      color: `${tipo}`,
    });
  }

  public deleteLinha({ row }: any) {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    grid.datasource.data.forEach((objeto: any) => {
      if (objeto.hora === '') {
        grid.datasource.data.splice(objeto, 1);
        grid.datasource.data = [];
        this.getDadosApontamentoInserido();
      } else if (objeto.hora !== '') {
        if (objeto.name === row.name) {
          Http.post('deleteRegistroHoras', { objeto }).then((response: any) => {
            if (response.data.dataset.dados.retorno === 'DELETADO') {
              const texto = 'Apontamento deletado com sucesso!';
              const sucesso = 'success';
              this.mensagemAlert(texto, sucesso);
              this.geraPrimeiroUltimoDiaMes();
              this.geraConteudoGrid();
              grid.datasource.data = [];
              this.getDadosApontamentoInserido();
            } else {
              const texto = 'Ocorreu um erro ao deletar o apontamento';
              const sucesso = 'error';
              this.mensagemAlert(texto, sucesso);
            }
          });
        }
      }
    });
  }

  public apurar() {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    const arrHoras: IDictionary = [];
    const parametrosUpdate: IDictionary = [];
    let entrada = '';
    let saida = '';
    let horasTotais = '';
    let entradaAlmoco = '';
    let saidaAlmoco = '';

    grid.datasource.data.forEach((dados: IDictionary) => {
      if (dados.tipoEntrada === '1' && dados.idAlmoco === 'N') {
        entrada = dados.hora;
        arrHoras.push(entrada);
      } else if (dados.tipoEntrada === '2' && dados.idAlmoco === 'N') {
        saida = dados.hora;
        arrHoras.push(saida);
      } else if (dados.tipoEntrada === '2' && dados.idAlmoco === 'S') {
        saidaAlmoco = dados.hora;
        arrHoras.push(saidaAlmoco);
      } else if (dados.tipoEntrada === '1' && dados.idAlmoco === 'S') {
        entradaAlmoco = dados.hora;
        arrHoras.push(entradaAlmoco);
      }
    });

    const primeiraEntradaDia = arrHoras[0];
    const ultimaSaidaDia = arrHoras[arrHoras.length - 1];
    const intervalo = this.calculoIntervalo(entradaAlmoco, saidaAlmoco);
    horasTotais = this.calculoHoraTotal(arrHoras);

    parametrosUpdate.push({
      DATA: this.dataReferencia,
      NRFUNCIONARIO: this.nrfuncionario,
      HRENTRADA: primeiraEntradaDia,
      HRSAIDA: ultimaSaidaDia,
      HRINTERVALO: intervalo,
      HRTRABALHADAS: horasTotais,
      HRAUSENCIA: this.ausencia,
      HRTOTAIS: horasTotais,
      IDFALTA: 'N',
      IDAPURADO: 'S',
    });

    Http.post('/apurarHoras', parametrosUpdate).then((response: any) => {
      if (response.data.dataset.dados.retorno === 'APURADO') {
        const texto = 'Registro apurado com sucesso!';
        const sucesso = 'success';
        this.mensagemAlert(texto, sucesso);
      } else {
        const texto = 'Ocorreu um erro ao apurar o registro';
        const sucesso = 'error';
        this.mensagemAlert(texto, sucesso);
      }
      this.geraPrimeiroUltimoDiaMes();
      this.geraConteudoGrid();
    });
  }

  public calculoIntervalo(entradaAlmoco: string, saidaAlmoco: string) {
    const h1 = entradaAlmoco.split(':');
    const h2 = saidaAlmoco.split(':');

    const EntrHora = Number(h1[0]);
    const EntrMin = Number(h1[1]);
    const SaiHora = Number(h2[0]);
    const SaiMin = Number(h2[1]);

    const entrada = this.minutos(EntrHora, EntrMin);
    const saida = this.minutos(SaiHora, SaiMin);

    const intervalo = this.formataIntervalo(entrada, saida);
    return intervalo;
  }

  public minutos(h: number, m: number) {
    const hora = h * 3600;
    const minutos = m * 60;
    return {
      h: hora,
      m: minutos,
    };
  }

  public formataIntervalo(entrada: any, saida: any) {
    const hora = ((entrada.h - saida.h) / 3600);
    const minuto = ((entrada.m - saida.m) / 60);
    let retorno = '';
    if (minuto <= 10) {
      retorno = `0${hora}:0${minuto}`;
    } else {
      retorno = `${hora}:${minuto}`;
    }
    return retorno;
  }

  public calculoHoraTotal(arr: IDictionary) {
    const arrHoras: IDictionary = [];
    arr.forEach((hora: string) => {
      const h = hora.split(':');
      arrHoras.push(this.minutos(Number(h[0]), Number(h[1])));
    });

    const Horas: IDictionary = [];
    const Minutos: IDictionary = [];

    for (let i = 0; i < arrHoras.length - 1; i += 2) {
      Horas[i] = arrHoras[i + 1].h - arrHoras[i].h;
    }

    for (let i = 0; i < arrHoras.length - 1; i += 2) {
      Minutos[i] = arrHoras[i + 1].m - arrHoras[i].m;
    }

    let totalHora = 0;
    Horas.forEach((h: any) => {
      totalHora += h;
    });

    let totalMinutos = 0;
    Minutos.forEach((m: any) => {
      totalMinutos += m;
    });

    const horaFormatada = this.formataHoraTotal(totalHora, totalMinutos);
    this.horasTotais = horaFormatada;
    return horaFormatada;
  }

  public formataHoraTotal(h: number, m: number) {
    const hora = (h / 3600);
    const minuto = (m / 60);
    let retorno = '';
    if (hora < 10) {
      retorno = `0${hora}:0${Math.abs(minuto)}`;
      if (Math.abs(minuto) < 10) {
        retorno = `0${hora}:0${Math.abs(minuto)}`;
      } else {
        retorno = `0${hora}:${Math.abs(minuto)}`;
      }
    } else {
      retorno = `${hora}:${Math.abs(minuto)}`;
      if (Math.abs(minuto) < 10) {
        retorno = `${hora}:0${Math.abs(minuto)}`;
      } else {
        retorno = `${hora}:${Math.abs(minuto)}`;
      }
    }
    return retorno;
  }

  public VisibilidadeInformacao({ row }: any) {
    return row.obs;
  }

  public geraPrimeiroUltimoDiaMes() {
    const date = new Date();
    const primeiroDia = new Date(date.getFullYear(), date.getMonth(), 1);
    const ultimoDia = new Date(date.getFullYear(), date.getMonth() + 1, 0);
    Metadata.getInstance<DateRange>('periodo').value = [this.formataData(primeiroDia), this.formataData(ultimoDia)];
  }

  public mostraTotalHorasModal() {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    const arrHoras: IDictionary = [];
    let entrada = '';
    let saida = '';
    let horasTotais = '';
    let entradaAlmoco = '';
    let saidaAlmoco = '';
    grid.datasource.data.forEach((dados: IDictionary) => {
      if (dados.tipoEntrada === '1' && dados.idAlmoco === 'N') {
        entrada = dados.hora;
        arrHoras.push(entrada);
      } else if (dados.tipoEntrada === '2' && dados.idAlmoco === 'N') {
        saida = dados.hora;
        arrHoras.push(saida);
      } else if (dados.tipoEntrada === '2' && dados.idAlmoco === 'S') {
        saidaAlmoco = dados.hora;
        arrHoras.push(saidaAlmoco);
      } else if (dados.tipoEntrada === '1' && dados.idAlmoco === 'S') {
        entradaAlmoco = dados.hora;
        arrHoras.push(entradaAlmoco);
      }
    });
    this.getIntervaloPadraoTrabalho();
    horasTotais = this.calculoHoraTotal(arrHoras);
    Metadata.getInstance<Text>('textModalNovoApontamento').text = `Total: ${horasTotais}`;
  }

  public controlaVisibilidadeApurar() {
    const grid = Metadata.getInstance<GridEditable>('gridModalNovoApontamento');
    if (grid.datasource.data.length >= 4) {
      Metadata.getInstance<Button>('apurarNovoApontamento').disabled = false;
    } else {
      Metadata.getInstance<Button>('apurarNovoApontamento').disabled = true;
    }
  }

  public getDadosObservacao() {
    let observacao = '';
    Http.post('/getDadosDocEObservacoes', this.nrapontamento).then((response: any) => {
      response.data.dataset.dados.forEach((dados: any) => {
        observacao = observacao.concat(dados.DSOBSREGISTRO, ',');
        Metadata.getInstance<Textarea>('textarea').value = `${observacao}`;
      });
    });
  }

  public async getIntervaloPadraoTrabalho() {
    const NRFUNCIONARIO = this.nrfuncionario;
    const arrHoras: IDictionary = [];
    await Http.post('/getIntervaloTrabalho', { NRFUNCIONARIO }).then((response: any) => {
      response.data.dataset.dados.forEach((dados: any) => {
        arrHoras.push({
          entrada: dados.HRENTRADAPADR,
          saida: dados.HRSAIDAPADR,
        });
      });
      this.calculaIntervaloPadrao(arrHoras);
    });
  }

  public calculaIntervaloPadrao(arr: IDictionary) {
    let retorno = '';
    const horaTotal = this.horasTotais.split(':');
    const calcHoraTotal = ((horaTotal[0]) as any * 3600);
    const calcMinutoTotal = ((horaTotal[1]) as any * 60);
    arr.forEach((hora: IDictionary) => {
      const hEntradaPadrao = hora.entrada.split(':');
      const hSaidaPadrao = hora.saida.split(':');

      const calcHora = (((calcHoraTotal - (((hSaidaPadrao[0] * 3600) - (hEntradaPadrao[0] * 3600) - 3600))) / 3600));
      const calcMinuto = ((calcMinutoTotal - ((hSaidaPadrao[1] * 60) - (hEntradaPadrao[1] * 60))) / 60);

      const h = Math.abs(calcHora);
      const m = Math.abs(calcMinuto);

      let hAux = 0;
      let mAux = 0;
      if (h < 10 && m < 10) {
        retorno = `0${h}:0${m}`;
      } else if (h >= 10 && m >= 10) {
        hAux = (h - 1);
        mAux = (m - 60);
        retorno = `0${Math.abs(hAux)}:${Math.abs(mAux)}`;
      } else if (h < 10 && m >= 10) {
        hAux = (h - 1);
        mAux = (m - 60);
        retorno = `0${Math.abs(hAux)}:${Math.abs(mAux)}`;
      } else if (h >= 10 && m < 10) {
        retorno = `${h}:0${m}`;
      }
    });

    this.ausencia = retorno;
  }

  public insereTextoModal() {
    this.textoModal = `Intervalo de Marcação - ${this.dataReferencia}`;
  }

  public criarObjetoDate(date: string) {
    const d = date.slice(0, 10);
    const data = d.split('/').reverse().join('-');
    const hora = date.slice(11, 17);
    const objDate = new Date(`${data} ${hora}`);
    return `${objDate}`;
  }

  public pegaValorCheck({ component }: any) {
    this.valueIdAlmoco = component.value;
  }

  public getConditionalStyle({ row }: IDictionary<any>) {
    let propriedade = '';
    if (row.finalSemana === 1) {
      propriedade = 'finalDeSemana';
    }
    return `${propriedade}`;
  }

  public refreshData({ component }: { component: Select }) {
    component.datasource.get();
  }
}
